/* dclibc.c -- funzioni principali */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <glib.h>
#include "dclibc.h"
#include "hubconn.h"

/* costantine della versione della libreria installata */
const guint dclibc_major_version = DCLIBC_MAJOR_VERSION;
const guint dclibc_minor_version = DCLIBC_MINOR_VERSION;
const guint dclibc_micro_version = DCLIBC_MICRO_VERSION;

static MainLoopFunc mainloop_func = NULL;
static MainLoopFunc mainloop_exit = NULL;
static GQueue * trash_stack = NULL;

extern void _dc_hub_disconnect_all(void);

/* confronta le versioni passate con quelle interne */
const gchar* dclibc_check_version(guint required_major,guint required_minor,guint required_micro)
{
	if(required_major < dclibc_major_version) return "DCLibC major version mismatch.";
	if(required_minor < dclibc_minor_version) return "DCLibC minor version mismatch.";
	if(required_micro < dclibc_micro_version) return "DCLibC micro version mismatch.";
	return NULL;
}

/* inizializza la libreria
 loop_function: funzione da richiamare allo start del loop della libreria */
gboolean dclibc_init(MainLoopFunc loop_func,MainLoopFunc loop_exit)
{
	dc_hub_conn_init();
	dc_search_init();
	mainloop_func = loop_func;
	mainloop_exit = loop_exit;
	return TRUE;
}

/* entra nel mainloop */
gboolean dclibc_main(void)
{
	if(mainloop_func == NULL) return FALSE;
	(mainloop_func)();
	return TRUE;
}

/* esce dal mainloop */
gboolean dclibc_main_quit(void)
{
	if(mainloop_exit == NULL) return FALSE;
	g_debug("Exiting from mainloop...");
	_dc_hub_disconnect_all();
	g_idle_add((GSourceFunc)mainloop_exit,NULL);
	return TRUE;
}

gpointer dclibc_trash_add(gpointer data)
{
	if(data != NULL)
	{
		g_debug("Adding %p to dclibc garbage..\n",data);
		if(g_queue_get_length(trash_stack)) g_idle_add((GSourceFunc)dclibc_trash_flush,NULL);
		g_queue_push_head(trash_stack,data);
	}
	return data;
}

/* libera gli elementi nello stack della spazzatura */
gboolean dclibc_trash_flush(void)
{
	if(g_queue_get_length(trash_stack) <= 0) return FALSE;
	g_debug("Flushing dclibc garbage... (%d)\n",g_queue_get_length(trash_stack));
	gpointer data = NULL;
	while((data = g_queue_pop_head(trash_stack)) != NULL) g_free(data);
	return TRUE;
}
