/* dcproto.c -- implementazione protocollo NMDC */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include "dcproto.h"
#include "dclibc.h"
#include "hub.h"
#include "userlist.h"
#include "search.h"

#define append_unparsed(x,y)	g_string_append(x->unparsed,y)
#define deconvert_message(x)	({ \
	char * __msg = dc_proto_convert_reserved(x); \
	if(__msg == NULL) __msg = g_strdup(x); \
	__msg; \
})

/* define private solo per dc_proto_parse_nick/oplist */
#define nicklist_part1()	{ \
	if(hub == NULL || command == NULL) return; \
	char * con = dc_proto_content(command); \
	if(con == NULL) return; \
	char ** list = split_userlist(con); \
	g_free(con); \
	if(list != NULL) \
	{ \
		userinfo_t * u = NULL; \
		unsigned int i=0; \
		while(list[i] != NULL) \
		{ \
			if(strlen(list[i]) > 0) \
			{

#define nicklist_part2()	\
			} \
			i++; \
		} \
		g_strfreev(list); \
	} \
}

#if 0
#define DC_PARSE_MYINFO		DC_PREFIX_MYINFO " %s %s <%[^>]>$ $%[^$]$%[^$]$%llu[^|]$|"
#define DC_PARSE_MYINFO_CLIENT1	DC_PREFIX_MYINFO " %s %s <%[^>]><%[^>]>$ $%[^$]$%[^$]$%llu[^|]$|"
#define DC_PARSE_MYINFO_CLIENT2	DC_PREFIX_MYINFO " %s %s %[^<]<%[^>]>$ $%[^$]$%[^$]$%llu[^|]$|"
#endif

#define DC_PARSE_SR		DC_PREFIX_SR " %s %[^\5]\5%llu %u/%u\5%[^(] (%[^:)]:%u)|"
#define DC_PARSE_SR_DIR		DC_PREFIX_SR " %s %[^\5]%[^(] (%[^:)]:%u)|"

#define DC_PARSE_SEARCH		DC_PREFIX_SEARCH " %[^:]:%s %c?%c?%llu?%c?%[^|]|"

static int reservedchar(unsigned char c);
static int chat_reservedchar(unsigned char c);
static char ** split_userlist(const char * content);
static void destroy_message(message_t * msg);
static void destroy_redirect(redirect_t * rd);
extern void destroy_search(search_t * se);
extern void notify_result(hub_t * hub,char * sr_packet);

/* parsa il buffer GString di input dell'hub <hub>
 restituisce FALSE se il buffer e` vuoto (dopo aver eliminato il comando appena parsato) */
gboolean dc_proto_parse(hub_t * hub)
{
	//g_debug("Parsing: \"%s\"",hub->conn->buffer->str);
	if(!hub->conn->buffer->len) goto parse_end;
	int i;
	gboolean unparsed = FALSE;
	char * response = NULL;
	char * part = NULL;
	for(i=0;i<3 && part == NULL;i++) part = dc_proto_extract(hub->conn->buffer->str,i);
	//g_debug("Current part: \"%s\"",part);
	if(part == NULL) goto parse_end;
		/** $Lock **/
		if(g_str_has_prefix(part,DC_PREFIX_LOCK))
		{
			g_debug("(dc_proto_parse): Lock entry received");
			if(hub->handshake) // siamo in handshake
			{
				response = dc_proto_parse_lock(hub,part);
			}
			else // boh... strano, una lock così, a buffo ;)
			{
				append_unparsed(hub,part);
				response = NULL;
			}
		}
		/** $HubName **/
		else if(g_str_has_prefix(part,DC_PREFIX_HUBNAME))
		{
			dc_proto_parse_hubname(hub,part);
			g_debug("(dc_proto_parse): setting hubname to %s",hub->hubname);
			HUB_NOTIFY(hub,NOTIFY_HUBNAME,hub->hubname,NULL);
		}
		/** $Hello **/
		else if(g_str_has_prefix(part,DC_PREFIX_HELLO))
		{
			g_debug("(dc_proto_parse): Hello received");
			char * nick = NULL;
			response = dc_proto_parse_hello(hub,part,&nick);
			if(hub->handshake && response != NULL) // siamo noi, continuiamo l'handshake
			{
				g_debug("(dc_proto_parse): proceeding with handshake");
				hub->handshake = FALSE; // finito l'handshake
			}
			else	// normale login di un utente
			{
				g_debug("(dc_proto_parse): user login: %s",nick);
			}
			// notifica in ogni caso (effetto DC++ ;)
			HUB_NOTIFY(hub,NOTIFY_JOIN,nick,g_free);
		}
		/** < > **/
		else if(g_str_has_prefix(part,DC_PREFIX_CHATMSG))
		{
			g_debug("(dc_proto_parse): Main chat message received");
			message_t * msg = g_malloc0(sizeof(message_t));
			dc_proto_parse_message(part,msg);
			HUB_NOTIFY(hub,NOTIFY_CHATMSG,msg,(GDestroyNotify)destroy_message);
		}
		/** $Quit **/
		else if(g_str_has_prefix(part,DC_PREFIX_QUIT))
		{
			char * nick = dc_proto_parse_quit(hub,part);
			g_debug("(dc_proto_parse): user logout: %s",nick);
			HUB_NOTIFY(hub,NOTIFY_PART,nick,g_free);
		}
		/** $MyINFO **/
		else if(g_str_has_prefix(part,DC_PREFIX_MYINFO))
		{
			g_debug("(dc_proto_parse): user information");
			char * nick = dc_proto_parse_myinfo(hub,part);
			if(nick != NULL) HUB_NOTIFY(hub,NOTIFY_USERINFO,nick,g_free);
		}
		/** $To: **/
		else if(g_str_has_prefix(part,DC_PREFIX_TO))
		{
			g_debug("(dc_proto_parse): private message");
			message_t * msg = g_malloc0(sizeof(message_t));
			dc_proto_parse_to(part,msg);
			HUB_NOTIFY(hub,NOTIFY_PRIVMSG,msg,(GDestroyNotify)destroy_message);
		}
		/** $GetPass **/
		else if(g_str_has_prefix(part,DC_PREFIX_GETPASS))
		{
			g_debug("(dc_proto_parse): password request");
			HUB_NOTIFY(hub,NOTIFY_ASKPASS,NULL,NULL);
		}
		/** $BadPass **/
		else if(g_str_has_prefix(part,DC_PREFIX_BADPASS))
		{
			g_debug("(dc_proto_parse): wrong password");
			HUB_NOTIFY(hub,NOTIFY_BADPASS,NULL,NULL);
		}
		/** $ValidateDenide **/
		else if(g_str_has_prefix(part,DC_PREFIX_VALIDATEDENIDE))
		{
			char * nick = dc_proto_content(part);
			if(nick != NULL)
			{
				g_debug("(dc_proto_parse): invalid nick %s",nick);
				HUB_NOTIFY(hub,NOTIFY_INVALIDNICK,nick,g_free);
			}
		}
		/** $HubIsFull **/
		else if(g_str_has_prefix(part,DC_PREFIX_HUBISFULL))
		{
			g_debug("(dc_proto_parse): hub is full");
			HUB_NOTIFY(hub,NOTIFY_HUBFULL,NULL,NULL);
		}
		/** $LogedIn **/
		else if(g_str_has_prefix(part,DC_PREFIX_LOGEDIN))
		{
			char * nick = dc_proto_parse_logedin(hub,part);
			if(nick != NULL)
			{
				g_debug("(dc_proto_parse): operator %s joins",nick);
				HUB_NOTIFY(hub,NOTIFY_OPJOIN,nick,g_free);
			}
		}
		/** $HubTopic **/
		else if(g_str_has_prefix(part,DC_PREFIX_HUBTOPIC))
		{
			dc_proto_parse_hubtopic(hub,part);
			g_debug("(dc_proto_parse): hub topic is \"%s\"",hub->hubtopic);
			if(hub->hubtopic != NULL) HUB_NOTIFY(hub,NOTIFY_HUBTOPIC,hub->hubtopic,NULL);
		}
		/** $NickList **/
		else if(g_str_has_prefix(part,DC_PREFIX_NICKLIST))
		{
			g_debug("(dc_proto_parse): parsing nicklist...");
			dc_proto_parse_nicklist(hub,part);
			HUB_NOTIFY(hub,NOTIFY_NICKLIST,NULL,NULL);
		}
		/** $OpList **/
		else if(g_str_has_prefix(part,DC_PREFIX_OPLIST))
		{
			g_debug("(dc_proto_parse): parsing oplist...");
			dc_proto_parse_oplist(hub,part);
			HUB_NOTIFY(hub,NOTIFY_OPLIST,NULL,NULL);
		}
		/** $ForceMove **/
		else if(g_str_has_prefix(part,DC_PREFIX_FORCEMOVE))
		{
			g_debug("(dc_proto_parse): forced redirect");
			redirect_t * rd = dc_proto_parse_forcemove(hub,part);
			if(rd != NULL)
			{
				// ci disconnettiamo tanto il server non accetta più comandi
				//dc_hub_disconnect(hub);
				// segnala al client che siamo stati dirottati
				HUB_NOTIFY(hub,NOTIFY_REDIRECT,rd,(GDestroyNotify)destroy_redirect);
			}
		}
		/** $SR **/
		else if(g_str_has_prefix(part,DC_PREFIX_SR))
		{
			g_debug("(dc_proto_parse): passive search result");
			notify_result(hub,part);
		}
		/** $Search **/
		else if(g_str_has_prefix(part,DC_PREFIX_SEARCH))
		{
			g_debug("(dc_proto_parse): search request");
			search_t * se = dc_proto_parse_search(part);
			if(se != NULL) HUB_NOTIFY(hub,NOTIFY_SEARCH_REQUEST,se,(GDestroyNotify)destroy_search);
		}
		else
		{
			//g_debug("######## unparsed: \"%s\"",part);
			append_unparsed(hub,part);
			response = NULL;
			//unparsed = TRUE;
		}
		if(response != NULL) dc_proto_send(hub,response);
		/*if(!unparsed)*/ g_string_erase(hub->conn->buffer,0,strlen(part));
		g_free(part);
		if(/* !unparsed && */ !hub->conn->buffer->len) goto parse_end; // non abbiamo piu` niente da parsare
	//g_debug("(dc_proto_parse): unparsed data: \"%s\"",hub->unparsed->str);
	g_debug("Other to parse, returning TRUE");
	return TRUE;	// abbiamo altro da parsare
parse_end:
	g_debug("Nothing to parse, returning FALSE");
	hub->parsetag = -1;
	return FALSE;
}

/* estrae il comando numero <command_nr> dalla stringa di comandi <commands>
 estrae anche i messaggi di main chat
 restituisce una stringa nuova galloccata, altrimenti NULL in caso di errore */
char * dc_proto_extract(const char * commands,unsigned int command_nr)
{
	#if 0
	unsigned int i;
	char * cmdcopy = g_strdup(commands);
	char *dollar = cmdcopy,*pipe = cmdcopy;
	dollar--;
	//if(want_debug) printf("Parsatura di \"%s\"\n",cmdcopy);
	for(i=0;i<=command_nr;i++)
	{
		dollar = strstr(++dollar,"$");
		//if(want_debug) printf("dollar = 0x%x\n",(unsigned int )dollar);
		//if(want_debug) printf("dollar: \"%s\"\n",dollar);
		if(dollar == NULL) return NULL;
		pipe = strstr(dollar,"|");
		//if(want_debug) printf("pipe = 0x%x\n",(unsigned int )pipe);
		if(pipe == NULL) return NULL;
	}
	//if(want_debug) printf("uscito dal for, i = %d, command_nr = %d\n",i,command_nr);
	if(i < command_nr) return NULL;
	cmdcopy = dollar;
	*++pipe = '\0';
	//if(want_debug) printf("COPY: \"%s\"\n",cmdcopy);
	return cmdcopy;
	#else
	unsigned int i;
	//if(want_debug) printf("Command_nr = %d, max_tokens = %d\n",command_nr,command_nr+3);
	char ** c = g_strsplit(commands,"|",command_nr+3);
	//if(want_debug) printf("c is 0x%x\n",(unsigned int)c);
	if(c == NULL) return NULL;
	for(i=0;i<command_nr && c[i] != NULL;i++) /*if(want_debug) printf("c[%d] = 0x%x\n",i,(unsigned int )c[i])*/;
	//if(want_debug) printf("END: c[%d] = 0x%x\n",i,(unsigned int )c[i]);
	if(c[i] == NULL || i > command_nr) return NULL;
	if(!strcmp(c[i],"")) return NULL;
	char * d = g_malloc0(strlen(c[command_nr])+2);
	strcpy(d,c[command_nr]);
	//if(want_debug) printf("SUBST: \"%s\"\n",d);
	char * e = &d[strlen(d)];
	*e = '|';
	*++e = '\0';
	g_strfreev(c);
	//if(want_debug) printf("EXTRACT: \"%s\"\n",d);
	return d;
	#endif
}

static int reservedchar(unsigned char c)
{
    return((c == 0) || (c == 5) || (c == 124) || (c == 96) || (c == 126) || (c == 36));
}

static int chat_reservedchar(unsigned char c)
{
    return((c == 124) || (c == 36));
}

/* genera la key dalla lock */
char * dc_proto_generate_key(char *lock)
{
    int i, len, offset;
    char *buf, *key;
    char save;
    
    /* Step 1: Compute key */
    buf = g_malloc0(strlen(lock));
    save = 5;
    len = 0;
    for(i = 0; lock[i]; i++) {
        buf[i] = lock[i] ^ save;
        buf[i] = ((buf[i] & 0x0F) << 4) | ((buf[i] & 0xF0) >> 4);
        save = lock[i];
        if((i != 0) && reservedchar(buf[i]))
            len += 10;
        else
            len++;
    }
    buf[0] ^= buf[i - 1];
    if(reservedchar(buf[0]))
        len += 10;
    else
        len++;
    
    /* Step 2: Quote reserved characters */
    key = g_malloc0(len + 1);
    offset = 0;
    for(i = 0; lock[i] != 0; i++) {
        if(reservedchar(buf[i]))
            offset += sprintf(key + offset, "/%%DCN%03i%%/", buf[i]);
        else
            key[offset++] = buf[i];
    }
    key[offset] = 0;
    g_free(buf);
    
    /* Observe: The caller will have to free the memory */
    return(key);
}

/* invia un comando su formattazione
 usare solo per invio di comandi, non per lo scambio di dati con un peer(la)
 restituisce il numero di byte inviati
 in realtà mette in coda i dati nella coda di output */
int dc_proto_send(hub_t * hub,const char * command,...)
{
	if(hub == NULL || command == NULL) return -1;
	/* controlliamo se siamo connessi */
	char * buf = NULL;
	va_list args;
	va_start(args,command);
	buf = g_strdup_vprintf(command,args);
	va_end(args);
	if(buf == NULL) buf = g_strdup(command);
	/* metti in coda al buffer di output */
	dc_hub_conn_append_data(hub->conn,buf);
	int c = strlen(buf);
	g_free(buf);
	return c;
}

/* estrae gli argomenti del comando <command>, ovvero quello che viene dopo il prefisso del comando */
char * dc_proto_content(const char * command)
{
	char * preinit = g_strdup(command);
	char * init = strstr(preinit,"$");
	if(init == NULL) // proviamo a levare solo il | finale e ritorniamo
	{
		if(preinit[strlen(preinit)-1] == '|') preinit[strlen(preinit)-1] = '\0';
		return preinit;
	}
	char * space = strstr((init+1)," ");
	if(space == NULL) return "";
	char * content = (space + 1);
	char * pipe = strstr(content,"|");
	if(pipe == NULL) return NULL;
	*pipe = '\0';
	char * ret = g_strdup(content);
	g_free(preinit);
	return ret;
}

/* parsa una lock
 restituisce il responso da inviare all'hub */
char * dc_proto_parse_lock(hub_t * hub,const char * command)
{
	if(hub == NULL || command == NULL) return NULL;
	char * copy = g_strdup(command);
	/* estrai la key */
	char * lock = strstr(copy,DC_PREFIX_LOCK " ");
	if(lock == NULL) return NULL;
	lock += strlen(DC_PREFIX_LOCK " ");
	char * end = strstr(lock," Pk=");
	if(end == NULL) return NULL;
	*end = '\0';
	g_debug("(dc_proto_parse_lock): generating key with lock '%s'",lock);
	char * key = dc_proto_generate_key(lock);
	g_free(copy);
	if(key == NULL) return NULL;
	g_debug("(dc_proto_parse_lock): making lock response with key '%s'",key);
	return g_strdup_printf(/* HACK */ "$Supports NoGetINFO|" DC_CMD_KEY DC_CMD_VALIDATENICK,key,hub->userinfo->nick);
}

/* parsa un hello
 se hub->handshake == TRUE e il nick del comando e` uguale al nostro, restituisce la stringa di responso
 altrimenti restituisce NULL */
char * dc_proto_parse_hello(hub_t * hub,const char * command,char ** nick)
{
	if(hub == NULL || command == NULL) return NULL;
	*nick = dc_proto_content(command);
	char * ret = NULL;
	if(hub->handshake == TRUE && !g_ascii_strcasecmp(hub->userinfo->nick,*nick))
	{
		char * cid = hub->userinfo->client;
		if(cid == NULL) cid = g_strdup("");
		char * dsc = hub->userinfo->desc;
		if(dsc == NULL) dsc = g_strdup("");
		if(hub->userinfo->client == NULL) g_free(cid);
		if(hub->userinfo->desc == NULL) g_free(dsc);
		char * cid_pass = g_strdup_printf("%s%s",dsc,cid);
		ret = g_strdup_printf(DC_CMD_VERSION DC_CMD_MYINFO DC_CMD_GETNICKLIST,DC_DEFAULT_VERSION,"$ALL",
		 hub->userinfo->nick,cid_pass,hub->userinfo->tag,hub->userinfo->speed,hub->userinfo->type,
		 hub->userinfo->email,hub->userinfo->share_size);
		g_free(cid_pass);
		userlist_add(hub->userlist,hub->userinfo);
	}
	else
	{
		/* aggiungi il nick alla userlist dell'hub */
		userlist_add(hub->userlist,DCTRASH(userinfo_new(*nick,NULL,NULL,NULL,NUM_SPEEDS,NORMAL,FALSE,0)));
	}
	//g_debug("hello: sending `%s`",ret);
	return ret;
}

/* parsa un hubname */
void dc_proto_parse_hubname(hub_t * hub,const char * command)
{
	if(hub != NULL && command != NULL) hub->hubname = dc_proto_content(command);
}

/* parsa un quit
 restituisce il nick */
char * dc_proto_parse_quit(hub_t * hub,const char * command)
{
	if(hub == NULL || command == NULL) return NULL;
	char * nick = dc_proto_content(command);
	/* rimuovi il nick dalla userlist dell'hub */
	userlist_remove(hub->userlist,nick);
	return nick;
}

/* parsa un to
 mette in <msg> con i dati da notificare */
void dc_proto_parse_to(const char * command,message_t * msg)
{
	char * pmsg = g_strdup(command);
	char * from = strstr(pmsg,"From: ");
	if(from == NULL) return;
	from += strlen("From: ");
	char * enduser = strstr(from," ");
	if(enduser == NULL) return;
	*--from = '<';
	*enduser = '>';
	*++enduser = ' ';
	char * ret = g_strdup(from);
	g_free(pmsg);
	//g_debug("(dc_proto_parse_to): Private message parsed: \"%s\"\n",ret);
	dc_proto_parse_message(ret,msg);
	g_free(ret);
}

/* parsa un logedin
 restituisce il nick */
char * dc_proto_parse_logedin(hub_t * hub,const char * command)
{
	char * nick = dc_proto_content(command);
	if(nick != NULL)
	{
		userinfo_t * u = (userinfo_t *)userlist_get_from_nick(hub->userlist,nick);
		if(u == NULL)
		{
			u = g_malloc0(sizeof(userinfo_t));
			u->nick = g_strdup(nick);
			u->is_op = TRUE;
			userlist_add(hub->userlist,u);
			userinfo_delete(u);
		}
		else u->is_op = TRUE;
	}
	return nick;
}

static void destroy_message(message_t * msg)
{
	if(msg != NULL)
	{
		g_free(msg->nick);
		g_free(msg->message);
		g_free(msg);
	}
}

static void destroy_redirect(redirect_t * rd)
{
	if(rd != NULL)
	{
		g_free(rd->server);
		g_free(rd);
	}
}

/* mette in <buffer> il nick e il messaggio da <message> in questo formato:
 <<nick>> <messaggio> */
gboolean dc_proto_parse_message(const char * message,message_t * buffer)
{
	if(message != NULL && buffer != NULL)
	{
		buffer->nick = dc_proto_extract_user(message);
		buffer->message = dc_proto_extract_message(message);
		return TRUE;
	}
	return FALSE;
}

/* restituisce una nuova stringa allocata con g_strdup contenente l'interno di < > */
char * dc_proto_extract_user(const char * buf)
{
	char * str = g_strdup(buf);
	char * start = strstr(str,"<");
	if(start == NULL)
	{
		g_free(str);
		return NULL;
	}
	char * end = strstr(start,">");
	if(end == NULL)
	{
		g_free(str);
		return NULL;
	}
	start++;
	*end = '\0';
	char * ret = g_strdup(start);
	g_free(str);
	return ret;
}

/* estra il testo del messaggio da un pvt o mainchat message */
char * dc_proto_extract_message(const char * buf)
{
	char * str = g_strdup(buf);
	char * start = strstr(str,"> ");
	if(start == NULL)
	{
		g_free(str);
		return NULL;
	}
	start += 2;
	if(start[strlen(start)-1] == '|') start[strlen(start)-1] = '\0';
	/* converti i caratteri riservati in caratteri normali */
	char * ret = dc_proto_deconvert_reserved(start);
	g_free(str);
	return ret;
}

/* converte i caratteri riservati
 restituisce la stringa convertita */
char * dc_proto_convert_reserved(const char * str)
{
	if(str == NULL) return NULL;
	int i, len=0, offset;
	char * ret;
	/* Passo 1: calcola lunghezza finale della stringa convertita */
	for(i = 0; str[i]; i++)
	{
		if(chat_reservedchar(str[i]))
		{
			len += 5;
			if(str[i] >= 100) len++;
		}
		else len++;
	}
	/* Passo 2: converti i caratteri riservati */
	ret = g_malloc0(len + 1);
	offset = 0;
	for(i = 0; str[i] != 0; i++)
	{
		if(chat_reservedchar(str[i])) offset += sprintf(ret + offset, "&#%i;", str[i]);
		else ret[offset++] = str[i];
	}
	ret[offset] = '\0';
	return(ret);
}

/* "de-converte" i caratteri riservati
 restituisce la stringa de-convertita */
char * dc_proto_deconvert_reserved(const char * str)
{
	if(str == NULL) return NULL;
	char * buf = g_strdup(str);
	char * ret = g_malloc0(strlen(str)+1);
	char old = '\0';
	unsigned int i,t=0,offset = 0;
	for(i=0;str[i];i++)
	{
		if(str[i] == '&' && str[i+1] == '#' && (g_ascii_isdigit(str[i+2]) == TRUE))
		{	// ok, abbiamo un sospetto carattere, cerca il punto e virgola dopo
			if(str[i+3] == ';') offset = 3;
			else if(str[i+4] == ';') offset = 4;
			else if(str[i+5] == ';') offset = 5;
			else offset = -1;
			if(offset >= 3)
			{	// ce l'abbiamo, e` un carattere speciale, convertilo nel suo valore ascii
				old = buf[i+offset];
				buf[i+offset] = '\0';
				ret[t++] = atoi(&buf[i+2]);
				buf[i+offset] = old;
				i += offset;
				continue;
			}
		}
		ret[t++] = str[i];	// copia il carattere originale
	}
	ret[t] = '\0';
	g_free(buf);
	return ret;
}

/* invia un messaggio in main all'hub <hub> */
void dc_proto_send_main_message(hub_t * hub,const char * message)
{
	if(hub != NULL && message != NULL)
	{
		char * msg = deconvert_message(message);
		dc_proto_send(hub,DC_CMD_CHATMSG,hub->userinfo->nick,msg);
		g_free(msg);
	}
}

/* invia un messaggio privato a <nick> nell'hub <hub> */
void dc_proto_send_priv_message(hub_t * hub,const char * nick,const char * message)
{
	if(hub != NULL && message != NULL)
	{
		char * msg = deconvert_message(message);
		dc_proto_send(hub,DC_CMD_TO,nick,hub->userinfo->nick,hub->userinfo->nick,msg);
		g_free(msg);
	}
}

/* caccia un utente dall'hub */
void dc_proto_kick(hub_t * hub,const char * nick)
{
	if(nick != NULL && hub != NULL) dc_proto_send(hub,DC_CMD_KICK,nick);
}

/* disconnette un utente dall'hub */
void dc_proto_close(hub_t * hub,const char * nick)
{
	if(nick != NULL && hub != NULL) dc_proto_send(hub,DC_CMD_CLOSE,nick);
}

/* invia la password */
void dc_proto_send_password(hub_t * hub,const char * pass)
{
	if(pass != NULL && hub != NULL) dc_proto_send(hub,DC_CMD_MYPASS,pass);
}

/* invia il comando di uscita */
void dc_proto_quit(hub_t * hub)
{
	if(hub != NULL) dc_proto_send(hub,DC_PREFIX_QUIT "|");
}

/* parsa un hubtopic */
void dc_proto_parse_hubtopic(hub_t * hub,const char * command)
{
	if(hub != NULL && command != NULL) hub->hubtopic = dc_proto_content(command);
}

/* parsa un myinfo
 restituisce il nick di cui sono arrivate le myinfo
 da aggiustare perché non si può proprio guardare :S */
char * dc_proto_parse_myinfo(hub_t * hub,const char * command)
{
	userinfo_t * u = NULL;
	char * client = NULL;
	// $MyINFO $ALL nick <tag>$ $speedt$email$sharesize$|
	char * con = dc_proto_content(command);
	if(con == NULL) return NULL;
	g_debug("MyINFO content: \"%s\"",con);
	/* estrai solo il nick perché necessario */
	char * dest = g_strdup(con);
	char * enddest = strstr(dest," ");
	if(enddest == NULL)
	{
		g_free(dest);
		g_free(con);
		return NULL;
	}
	*enddest = '\0';
	g_debug("(dc_proto_parse_myinfo): extracted destination: \"%s\"",dest);
	char * tnick = g_strdup(enddest+1);
	char * endnick = strstr(tnick," ");
	if(endnick == NULL)
	{
		g_free(tnick);
		g_free(dest);
		g_free(con);
		return NULL;
	}
	*endnick = '\0';
	char * nick = g_strdup(tnick);
	g_free(tnick);
	g_debug("(dc_proto_parse_myinfo): extracted nick: \"%s\"",nick);
	// estrai client + tag
	char * ttag = g_strdup(con + strlen(dest) + 1 + strlen(nick) + 1);
	char * endtag = strstr(ttag,"$");
	if(endtag == NULL)
	{
		g_free(nick);
		g_free(ttag);
		g_free(dest);
		g_free(con);
		return NULL;
	}
	*endtag = '\0';
	char * tag = g_strdup(ttag);
	g_free(ttag);
	g_debug("(dc_proto_parse_myinfo): extracted client+tag: \"%s\"",tag);
	char * bak = tag;
	ttag = g_strrstr(tag,"<");	// cerca l'ultimo '<' nella stringa
	char * match_tag1 = g_strrstr(tag,"V:");
	char * match_tag2 = g_strrstr(tag,"H:");
	char * match_tag3 = g_strrstr(tag,"M:");
	char * match_tag4 = g_strrstr(tag,"S:");
	g_debug("(dc_proto_parse_myinfo): ttag = %p, match_tag1 = %p",ttag,match_tag1);
	if(ttag == NULL)	// tieni unito, niente tag
	{
		client = tag;
		tag = g_strdup("");
	}
	else
	{
		if((match_tag1 > ttag) && (match_tag2 > ttag) && (match_tag3 > ttag) && (match_tag4 > ttag))
		{
			tag = g_strdup(ttag+1);
			tag[strlen(tag)-1] = '\0';
			*ttag = '\0';
			client = g_strdup(bak);
			g_free(bak);
		}
		else
		{	// anche in questo caso tieni unito, niente tag
			client = tag;
			tag = g_strdup("");
		}
	}
	g_debug("(dc_proto_parse_myinfo): extracted client: \"%s\"",client);
	g_debug("(dc_proto_parse_myinfo): extracted tag: \"%s\"",tag);
	int factor = (strlen(tag) > 0)?2:0;
	// estrai la speed e la type
	char * tspeed = g_strdup(con + strlen(dest) + 1 + strlen(nick) + 2 + strlen(client) + strlen(tag) + 2 + factor);
	//g_debug("tspeed = `%s`",tspeed);
	char * endspeed = strstr(tspeed,"$");
	char * speed,type = '\0';
	int pfactor = 0;
	if(endspeed == NULL)
	{
		speed = g_strdup("");
		type = 0;
		/*
		g_free(nick);
		g_free(tag);
		g_free(tspeed);
		g_free(dest);
		g_free(con);
		return NULL;
		*/
	}
	else
	{
		if(endspeed > tspeed)
		{
			type = *--endspeed;
			pfactor = 1;
		}
		*endspeed = '\0';
		speed = g_strdup(tspeed);
	}
	g_free(tspeed);
	g_debug("(dc_proto_parse_myinfo): extracted speed: \"%s\" type = %d",speed,type);
	// estrai l'email
	// $MyINFO $ALL nick <tag>$ $speedt$$sharesize$|
	char * temail = g_strdup(con + strlen(dest) + 1 + strlen(nick) + 2 + strlen(tag) + strlen(client) + 2 + factor + strlen(speed) + 1 + pfactor);
	//g_debug("temail = `%s`",temail);
	char * endemail = strstr(temail,"$");
	//g_debug("endemail = %p",endemail);
	char * email;
	if(endemail == NULL)
	{
		email = g_strdup("");
		/*
		g_free(nick);
		g_free(tag);
		g_free(speed);
		g_free(temail);
		g_free(dest);
		g_free(con);
		return NULL;
		*/
	}
	else
	{
		*endemail = '\0';
		email = g_strdup(temail);
	}
	g_free(temail);
	g_debug("(dc_proto_parse_myinfo): extracted email: \"%s\"",email);
	// estrai la sharesize
	char * tshare = g_strdup(con + strlen(dest) + 1 + strlen(nick) + 2 + strlen(tag) + strlen(client) + 2 + factor + strlen(speed) + 1 + pfactor + strlen(email) + 1);
	char * endshare = strstr(tshare,"$");
	if(endshare == NULL)
	{
		strcpy(tshare,"0");
		/*
		g_free(nick);
		g_free(tag);
		g_free(speed);
		g_free(email);
		g_free(dest);
		g_free(con);
		return NULL;
		*/
	}
	else
	{
		*endshare = '\0';
	}
	g_debug("(dc_proto_parse_myinfo): extracted share size: \"%s\" (%d)",tshare,strlen(tshare));
	char * err = NULL;
	unsigned long long share_size = g_ascii_strtoull(tshare,&err,10);
	if(*err != 0)
	{
		g_debug("ERROR!!! err = \"%s\", share_size = %llu\n",err,share_size);
		g_free(tshare);
		g_free(nick);
		g_free(tag);
		g_free(speed);
		g_free(email);
		g_free(dest);
		g_free(con);
		return NULL;
	}
	g_free(tshare);
	g_debug("(dc_proto_parse_myinfo): converted share size: %llu",share_size);
	g_free(dest);
	g_free(con);
	/* aggiorna la userlist con i nuovi dati */
	u = g_malloc0(sizeof(userinfo_t));
	u->nick = nick;
	u->desc = g_strdup("");
	u->client = client;
	u->tag = tag;
	u->email = email;
	u->type = type;
	u->speed = speed;
	u->is_shared = TRUE;
	u->share_size = share_size;
	/* guarda se ci sono vecchie info, giusto per i parametri non riampazzabili dalla tag come is_op */
	const userinfo_t * t = userlist_get_from_nick(hub->userlist,nick);
	if(t != NULL) u->is_op = t->is_op;
	else u->is_op = FALSE; // inizialmente non è un operatore
	userlist_update(hub->userlist,u);
	nick = g_strdup(nick);
	userinfo_delete(u);
	return nick;
}

void dc_proto_get_nicklist(hub_t * hub)
{
	if(hub != NULL) dc_proto_send(hub,DC_CMD_GETNICKLIST);
}

/* parsa una nicklist */
void dc_proto_parse_nicklist(hub_t * hub,const char * command)
{
	nicklist_part1();
	u = g_malloc0(sizeof(userinfo_t));
	u->nick = g_strdup(list[i]);
	userlist_add(hub->userlist,u);
	userinfo_delete(u);
	nicklist_part2();
}

/* parsa una oplist */
void dc_proto_parse_oplist(hub_t * hub,const char * command)
{
	nicklist_part1();
	u = (userinfo_t *)userlist_get_from_nick(hub->userlist,list[i]);
	g_debug("(oplist): u = %p",u);
	if(u != NULL) u->is_op = TRUE;
	else
	{
		u = g_malloc0(sizeof(userinfo_t));
		u->nick = g_strdup(list[i]);
		u->is_op = TRUE;
		userlist_add(hub->userlist,u);
		userinfo_delete(u);
		u = (userinfo_t *)userlist_get_from_nick(hub->userlist,list[i]);
		g_debug("MA LOOL: nuovo nick: \"%s\"",u->nick);
	}
	nicklist_part2();
}

/* restituisce un'array di stringhe ottenuta dallo split di <content> con "$$" */
static char ** split_userlist(const char * content)
{
	if(content != NULL) return g_strsplit(content,"$$",-1);
	else return NULL;
}

/* forza la redirezione di un client
 bisogna essere op per usare questo comando */
void dc_proto_opforcemove(hub_t * hub,const char * victim,const char * server,const char * reason)
{
	if(victim != NULL && server != NULL && reason != NULL)
	 dc_proto_send(hub,DC_CMD_OPFORCEMOVE,victim,server,reason);
}

/* parsa un forcemove
 restituisce il server dove siamo rediretti sotto forma di struct redirect_t */
redirect_t * dc_proto_parse_forcemove(hub_t * hub,const char * command)
{
	char * server = dc_proto_content(command);
	server = g_strstrip(server);
	redirect_t * rd = NULL;
	if(server != NULL)
	{
		/* separa host ed eventuale porta */
		char * port_off = strstr(server,":");
		char * port_err = NULL;
		int dport = 0;
		if(port_off == NULL) dport = 411; // ma porca...
		else
		{
			port_off++;
			dport = strtol(port_off,&port_err,10);
			if(port_err != NULL) if(*port_err != '\0') dport = 411; // ma porca...
			port_off[-1] = '\0';
		}
		g_debug("(dc_proto_parse_forcemove): redirecting to \"%s\" port %d...",server,dport);
		rd = g_malloc0(sizeof(redirect_t));
		rd->server = g_strdup(server);
		rd->port = dport;
		g_free(server);
	}
	return rd;
}

/* effettua una ricerca sull'hub <hub>
 size_restricted: indica se la ricerca e` ristretta alla dimensione
 is_max_size: indica se la dimensione indicata e` la massima, altrimenti la minima (valido se size_restricted == TRUE)
 size: indica la dimensione a cui e` limitata la ricerca (valido se size_restricted == TRUE)
 pattern: stringa di ricerca (i caratteri non alfa-numerici verranno convertiti in '$')
 restituisce l'id della ricerca */
int dc_proto_search(hub_t * hub,gboolean size_restricted,gboolean is_max_size,guint64 size,DATA_TYPE type,const char * pattern)
{
	if(pattern == NULL) return -1;
	char * conv = g_strdup(pattern);
	int c = strlen(conv);
	unsigned int i;
	for(i=0;i<c;i++) if(!isalnum((int )conv[i])) conv[i] = '$';
	int res = dc_search_request(hub,size_restricted,is_max_size,size,type,conv);
	g_free(conv);
	return res;
}

static gboolean str_is_number(const char * str)
{
	int i;
	for(i=0;i<strlen(str);i++)
	{
		g_debug("str[%d] = '%c'",i,str[i]);
		g_debug("is %d",g_ascii_isdigit(str[i]));
		if(g_ascii_isdigit(str[i]) == FALSE) return FALSE;
	}
	return TRUE;
}

/* parsa un search
 restituisce una struct search_t con i dati della ricerca richiesti */
// "$Search %[^:]:%s %c?%c?%llu?%c?%[^|]|"
search_t * dc_proto_parse_search(const char * command)
{
	char * nickI = g_malloc0(100);	// attivo: ip, passivo: "Hub"
	char * nickII = g_malloc0(100);	// attivo: porta, passivo: nick
	unsigned int port = 0;
	char size_restricted = '\0',is_max_size = '\0';
	unsigned long long size = 0;
	char datatype = '\0';
	char * pattern = g_malloc0(500);
	int ret = sscanf(command,DC_PARSE_SEARCH,nickI,nickII,&size_restricted,&is_max_size,&size,&datatype,pattern);
	if(ret < 7)
	{
		g_free(nickI);
		g_free(nickII);
		g_free(pattern);
		return NULL;
	}
	/*
	g_debug("(dc_proto_parse_search): nickI = \"%s\"",nickI);
	g_debug("(dc_proto_parse_search): nickII = \"%s\"",nickII);
	g_debug("(dc_proto_parse_search): size_restricted = '%c'",size_restricted);
	g_debug("(dc_proto_parse_search): is_max_size = '%c'",is_max_size);
	g_debug("(dc_proto_parse_search): size = %llu",size);
	g_debug("(dc_proto_parse_search): datatype = '%c'",datatype);
	g_debug("(dc_proto_parse_search): pattern = \"%s\"",pattern);
	*/
	//g_debug("strcmp = %d",strcmp(nickI,"Hub"));
	//g_debug("str_is_number = %d",str_is_number(nickII));
	if(!strcmp(nickI,"Hub") && !str_is_number(nickII))	// abbiamo una ricerca passiva: "Hub:<nick>"
	{
		//g_debug("(dc_proto_parse_search): not an IP, processing passive search");
		g_free(nickI);
		nickI = NULL;
		nickI = nickII;
	}
	else	// ricerca attiva: "<host>:<port>" (nickI:nickII)
	{
		//g_debug("(dc_proto_parse_search): IP found, processing active search");
		ret = sscanf(nickII,"%d",(int *)&port);
		g_free(nickII);
		if(ret < 1)
		{
			g_free(nickI);
			g_free(pattern);
			return NULL;
		}
		if(port < 1 || port > 65536)	// ammazza che merda!
		{
			g_free(nickI);
			g_free(pattern);
			return NULL;
		}
	}
	search_t * sr = g_malloc0(sizeof(search_t));
	//g_debug("(dc_proto_parse_search): === host_nick = \"%s\"",nickI);
	//g_debug("(dc_proto_parse_search): === port = %u",port);
	sr->host_nick = nickI;
	sr->port = port;
	sr->size_restricted = (size_restricted == 'T' || size_restricted == 't')?TRUE:FALSE;
	sr->is_max_size = (is_max_size == 'T' || is_max_size == 't')?TRUE:FALSE;
	sr->size = size;
	sr->type = datatype;
	sr->pattern = pattern;
	return sr;
}

/* parsa un sr
 restituisce una struct result_t con i dati della ricerca trovati*/
result_t * dc_proto_parse_sr(const char * command)
{
	if(command == NULL) return NULL;
	char * folder_slots = NULL;
	#if 0
	char * bak = NULL;
	char * cmd = g_strdup(command);
	// estrai il nick di chi ha il file
	char * nick = g_strdup(cmd + strlen(DC_PREFIX_SR) + 1);
	char * _end = strstr(nick," ");
	if(_end == NULL)
	{
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	*_end = '\0';
	bak = nick;
	nick = g_strdup(bak);
	g_free(bak);
	g_debug("(dc_proto_parse_sr): extracted nick: \"%s\"",nick);
	// estrai il nome del file
	char * file = g_strdup(cmd + strlen(DC_PREFIX_SR) + 1 + strlen(nick) + 1);
	_end = strstr(file,"\x05");
	if(_end == NULL)
	{
		g_free(file);
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	*_end = '\0';
	bak = file;
	file = g_strdup(bak);
	g_free(bak);
	/* controlla se e` una cartella (abbiamo gli slot al posto della size senza il 5... insomma un casino) */
	// cartella: $SR nick filepath F/T§hubname (hubhost:hubport)|
	//     file: $SR nick filepath§filesize F/T§hubname (hubhost:hubport)|
	/* se e` una cartella dobbiamo restringere il filename e memorizzare da qualche parte gli slot da elaborare
	 in seguito
	  se e` un file proseguiamo come se niente fosse, con folder_slots = NULL
	*/
	g_debug("(dc_proto_parse_sr): pre-extracted file: \"%s\"",file);
	// controlla se abbiamo lo slash da qualche parte
	int i;
	g_debug("strlen(file) = %d",strlen(file));
	for(i=strlen(file)-1;i>=0;i--)	// cerchiamo il primo spazio dalla fine della stringa
	{
		//g_debug("strlen(file) = %d, i = %d",strlen(file),i);
		if(file[i] == '/')	// abbiamo lo slash, cerchiamo lo spazio
		{
			g_debug("(dc_proto_parse_sr): slash found at char %d",i);
			int c;
			for(c=i-1;c>=0;c--)
			{
				if(file[c] == ' ')	// abbiamo lo spazio, prendiamo gli slot
				{
					g_debug("(dc_proto_parse_sr): space before slash found at char %d",c);
					folder_slots = g_strdup(&file[c+1]);
					g_debug("(dc_proto_parse_sr): extracted slots: \"%s\"",folder_slots);
					file[c] = '\0';
					break;
				}
			}
			break;
		}
	}
	g_debug("(dc_proto_parse_sr): extracted file: \"%s\"",file);
	char * _size = NULL;
	unsigned long long size = 0;
	char * err = NULL;
	if(folder_slots == NULL)
	{
		// estrai la dimensione del file
		_size = g_strdup(cmd + strlen(DC_PREFIX_SR) + 1 + strlen(nick) + 1 + strlen(file) + 1);
		_end = strstr(_size," ");
		if(_end == NULL)
		{
			g_free(_size);
			g_free(file);
			g_free(nick);
			g_free(cmd);
			return NULL;
		}
		*_end = '\0';
		g_debug("(dc_proto_parse_sr): extracted size: \"%s\"",_size);
		err = NULL;
		size = strtoull(_size,&err,10);
		if(*err != 0)
		{
			g_debug("ERROR!!! err = \"%s\", size = %llu\n",err,size);
			g_free(_size);
			g_free(file);
			g_free(nick);
			g_free(cmd);
			return NULL;
		}
		g_debug("(dc_proto_parse_sr): converted size: %llu",size);
	}
	// estrai gli slot disponibili
	char * _fslots = NULL;
	if(folder_slots != NULL)
	{
		_fslots = g_strdup(folder_slots);
	}
	else
	{
		_fslots = g_strdup(cmd + strlen(DC_PREFIX_SR) +1+ strlen(nick) +1+ strlen(file) +1+ strlen(_size) +1);
	}
	_end = strstr(_fslots,"/");
	if(_end == NULL)
	{
		g_free(_fslots);
		g_free(_size);
		g_free(file);
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	*_end = '\0';
	g_debug("(dc_proto_parse_sr): extracted free slots: \"%s\"",_fslots);
	err = NULL;
	unsigned int fslots = strtoul(_fslots,&err,10);
	if(*err != 0)
	{
		g_debug("ERROR!!! err = \"%s\", fslots = %lu\n",err,(unsigned long)fslots);
		g_free(_fslots);
		g_free(_size);
		g_free(file);
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	g_debug("(dc_proto_parse_sr): converted free slots: %lu",(unsigned long)fslots);
	// estrai gli slot totali
	char * _tslots = NULL;
	if(folder_slots != NULL)
	{
		_tslots = g_strdup(folder_slots);
		g_free(folder_slots);
		_end = &_tslots[strlen(_tslots)];
	}
	else
	{
		_tslots = g_strdup(cmd + strlen(DC_PREFIX_SR) +1+ strlen(nick) +1+ strlen(file) +1+ strlen(_size) +1 + strlen(_fslots) + 1);
		g_debug("(dc_proto_parse_sr): _tslots = \"%s\"",_tslots);
		_end = strstr(_tslots,"\x05");
	}
	if(_end == NULL)
	{
		g_free(_tslots);
		g_free(_fslots);
		g_free(_size);
		g_free(file);
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	*_end = '\0';
	g_debug("(dc_proto_parse_sr): extracted total slots: \"%s\"",_tslots);
	err = NULL;
	unsigned int tslots = strtoul(_tslots,&err,10);
	if(*err != 0)
	{
		g_debug("ERROR!!! err = \"%s\", tslots = %lu\n",err,(unsigned long)tslots);
		g_free(_tslots);
		g_free(_fslots);
		g_free(_size);
		g_free(file);
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	g_debug("(dc_proto_parse_sr): converted total slots: %lu",(unsigned long)tslots);
	// estrai hubname/TTH
	char * hubname = g_strdup(cmd + strlen(DC_PREFIX_SR) +1+ strlen(nick) +1+ strlen(file) +1+ strlen(_size) +1 + strlen(_fslots) + 1 + strlen(_tslots) +1);
	_end = strstr(hubname," (");
	if(_end == NULL)
	{
		g_free(hubname);
		g_free(_tslots);
		g_free(_fslots);
		g_free(_size);
		g_free(file);
		g_free(nick);
		g_free(cmd);
		return NULL;
	}
	*_end = '\0';
	g_debug("(dc_proto_parse_sr): extracted hubname/tth: \"%s\"",hubname);
	char * tth = NULL;
	bak = hubname;
	if(g_str_has_prefix(hubname,"TTH:"))
	{
		tth = g_strdup(hubname + strlen("TTH:"));
		hubname = NULL;
	}
	else hubname = g_strdup(hubname);
	g_free(bak);
	// estrai hubhost
	char * _hubport = NULL;	// per dopo
	char * hubhost = cmd + strlen(DC_PREFIX_SR) +1+ strlen(nick) +1+ strlen(file) +1+ strlen(_size) +1 + strlen(_fslots) + 1 + strlen(_tslots) +1;
	if(hubname != NULL) hubhost += strlen(hubname);
	else hubhost += strlen("TTH:") + strlen(tth);
	hubhost = g_strdup(hubhost + 2);
	_end = strstr(hubhost,":");
	if(_end == NULL)
	{
		g_debug("### port not found");
		_end = strstr(hubhost,")");
		if(_end == NULL)
		{
			g_debug("### closed pharentes not found");
			g_free(hubhost);
			g_free(hubname);
			g_free(_tslots);
			g_free(_fslots);
			g_free(_size);
			g_free(file);
			g_free(nick);
			g_free(cmd);
			return NULL;
		}
	}
	else _hubport = g_strdup(_end + strlen(":"));
	*_end = '\0';
	bak = hubhost;
	hubhost = g_strdup(bak);
	g_free(bak);
	g_debug("(dc_proto_parse_sr): extracted hubhost: \"%s\"",hubhost);
	// converti hubport (gia` estratta)
	port_t hubport = 411;
	if(_hubport != NULL)
	{
		_hubport[strlen(_hubport)-1] = '\0';
		//g_debug("### in-hubport: hubhost = \"%s\", _hubport = \"%s\"",hubhost,_hubport);
		err = NULL;
		hubport = strtoul(_hubport,&err,10);
		g_free(_hubport);
		if(hubport == 0 && err != _hubport)
		{
			g_debug("ERROR!!! err = \"%s\", hubport = %lu\n",err,(unsigned long)hubport);
			hubport = 411;
		}
	}
	g_debug("(dc_proto_parse_sr): converted hubport: %lu",(unsigned long)hubport);
	g_free(_size);
	g_free(_fslots);
	g_free(_tslots);
	#else
	char * nick = g_malloc0(100);
	char * file = g_malloc0(500);
	unsigned long long size = 0;
	unsigned int fslots = 0;
	unsigned int tslots = 0;
	char * hubname = g_malloc0(500);
	char * tth = NULL;
	char * hubhost = g_malloc0(100);
	port_t hubport = 0;
	int ret = sscanf(command,DC_PARSE_SR,nick,file,&size,&fslots,&tslots,hubname,hubhost,&hubport);
	g_debug("test: ret = %d",ret);
	g_debug("### nick = \"%s\"",nick);
	g_debug("### file = \"%s\"",file);
	g_debug("### hubname = \"%s\"",hubname);
	g_debug("### hubhost = \"%s\"",hubhost);
	g_debug("### hubport = %u",hubport);
	// abbiamo fallito se siamo a meno di 7 elementi (la porta puo` non esserci)
	if(ret < 7)	// proviamo con la directory
	{
		ret = sscanf(command,DC_PARSE_SR_DIR,nick,file,hubname,hubhost,&hubport);
		g_debug("DIRECTORY: ret = %d",ret);
		g_debug("nick = \"%s\"",nick);
		g_debug("file = \"%s\"",file);
		g_debug("hubname = \"%s\"",hubname);
		g_debug("hubhost = \"%s\"",hubhost);
		g_debug("hubport = %u",hubport);
		// anche qui se siamo a meno di 4 poiché la porta può non esserci
		if(ret < 4)
		{
			g_free(nick);
			g_free(file);
			g_free(hubname);
			g_free(hubhost);
			return NULL;
		}
		// estrai gli slot
		// controlla se abbiamo lo slash da qualche parte
		int i;
		g_debug("strlen(file) = %d",strlen(file));
		for(i=strlen(file)-1;i>=0;i--)	// cerchiamo il primo spazio dalla fine della stringa
		{
			//g_debug("strlen(file) = %d, i = %d",strlen(file),i);
			if(file[i] == '/')	// abbiamo lo slash, cerchiamo lo spazio
			{
				g_debug("(dc_proto_parse_sr): slash found at char %d",i);
				int c;
				for(c=i-1;c>=0;c--)
				{
					if(file[c] == ' ')	// abbiamo lo spazio, prendiamo gli slot
					{
						g_debug("(dc_proto_parse_sr): space before slash found at char %d",c);
						folder_slots = g_strdup(&file[c+1]);
						g_debug("(dc_proto_parse_sr): extracted slots: \"%s\"",folder_slots);
						file[c] = '\0';
						break;
					}
				}
				break;
			}
		}
		if(folder_slots == NULL)
		{
			g_free(nick);
			g_free(file);
			g_free(hubname);
			g_free(hubhost);
			return NULL;
		}
		ret = sscanf(folder_slots,"%u/%u",&fslots,&tslots);
		g_debug("free_slots = %u",fslots);
		g_debug("total_slots = %u",tslots);
	}
	else
	{
		g_debug("FILE: ret = %d",ret);
		g_debug("nick = \"%s\"",nick);
		g_debug("file = \"%s\"",file);
		g_debug("free_slots = %u",fslots);
		g_debug("total_slots = %u",tslots);
		g_debug("size = %llu",size);
		g_debug("hubname = \"%s\"",hubname);
		g_debug("hubhost = \"%s\"",hubhost);
		g_debug("hubport = %u",hubport);
	}
	hubname[strlen(hubname)-1] = '\0';
	g_debug("(dc_proto_parse_sr): extracted hubname/tth: \"%s\"",hubname);
	if(g_str_has_prefix(hubname,"TTH:"))
	{
		tth = g_strdup(hubname + strlen("TTH:"));
		g_free(hubname);
		hubname = NULL;
	}
	#endif
	result_t * r = g_malloc0(sizeof(result_t));
	r->nick = g_strdup(nick);
	g_free(nick);
	r->filename = g_strdup(file);
	g_free(file);
	r->filesize = size;
	r->free_slots = fslots;
	r->total_slots = tslots;
	r->extendedprotocol.hubname = g_strdup(hubname);
	g_free(hubname);
	r->extendedprotocol.tth = g_strdup(tth);
	g_free(tth);
	r->hubhost = g_strdup(hubhost);
	g_free(hubhost);
	r->hubport = hubport;
	r->is_folder = (folder_slots != NULL) ? TRUE : FALSE;
	g_free(folder_slots);
	return r;
}

/* richiede le myinfo per un utente */
void dc_proto_get_info(hub_t * hub,const char * nick)
{
	if(hub != NULL && nick != NULL)
	{
		g_debug("(dc_proto_get_info): requesting info for `%s`",nick);
		dc_proto_send(hub,DC_CMD_GETINFO,nick,hub->userinfo->nick);
	}
}
