/* userinfo.c -- gestione info utente */
/******************************************************************************
* DCLibC - Direct Connect C library                                           *
*                                                                             *
* Copyright (C) 2005-2007 daniele_athome                                      *
*                                                                             *
* This program is free software; you can redistribute it and/or               *
* modify it under the terms of the GNU General Public License                 *
* as published by the Free Software Foundation; either version 2              *
* of the License, or (at your option) any later version.                      *
*                                                                             *
* This program is distributed in the hope that it will be useful,             *
* but WITHOUT ANY WARRANTY; without even the implied warranty of              *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
* GNU General Public License for more details.                                *
*                                                                             *
* You should have received a copy of the GNU General Public License           *
* along with this program; if not, write to the Free Software                 *
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
*******************************************************************************/

#include <string.h>
#include <glib.h>
#include "dclibc.h"
#include "userinfo.h"

/* stringhe delle velocità */
const char * speed_types[NUM_SPEEDS] = {
	"Modem",
	"28.8Kbps",
	"33.6Kbps",
	"56Kbps",
	"Satellite",
	"ISDN",
	"DSL",
	"Cable",
	"LAN(T1)",
	"LAN(T3)"
};

#define TAG_WITHOUT_O	"%s V:%s,M:%c,H:%u/%u/%u,S:%u"
#define TAG_WITH_O	TAG_WITHOUT_O ",O:%u"

/* crea una nuova userinfo */
userinfo_t * userinfo_new(const char * nick,const char * email,const char * desc,const char * client_name,SPEED speed,CLIENT_TYPE type,gboolean is_shared,unsigned long long share_size)
{
	if(nick == NULL || type < 0 || type >= NUM_CLIENTS) return NULL;
	userinfo_t * u = g_malloc0(sizeof(userinfo_t));
	//g_debug("(userinfo_new): creating new userinfo in %p...",u);
	u->nick = g_strdup(nick);
	if(email == NULL) u->email = g_strdup("");
	else u->email = g_strdup(email);
	if(speed < NUM_SPEEDS) u->speed = g_strdup(speed_types[speed]);
	else u->speed = g_strdup("");
	if(desc == NULL) u->desc = g_strdup("");
	else u->desc = g_strdup(desc);
	u->type = type;
	if(is_shared == TRUE)
	{
		u->is_shared = TRUE;
		u->share_size = share_size;
	}
	if(client_name == NULL) u->client = g_strdup("");
	else u->client = g_strdup(client_name);
	return u;
}

/* distrugge una userinfo */
void userinfo_delete(userinfo_t * userinfo)
{
	g_free(userinfo->nick);
	g_free(userinfo->email);
	g_free(userinfo->speed);
	g_free(userinfo->client);
	g_free(userinfo->desc);
	g_free(userinfo->tag);
	g_free(userinfo);
	userinfo = NULL;
}

/* imposta una tag per la userinfo
 client_name: identificativo del client
 client_version: versione del client
 mode: TAG_MODE_ACTIVE o TAG_MODE_PASSIVE
 hubs_normal: numero di hub connessi con livello normale
 hubs_reg_vip: numero di hub connessi con livello vip o reg
 hubs_op: numero di hub connessi con livello op
 slots: numero totale di slot aperti
 speed_below_slot: velocità al di sotto della quale aprire uno slot aggiuntivo (a che serve non lo so :) */
void userinfo_set_tag(userinfo_t * userinfo,const char * client_name,const char * client_version,guint8 mode,guint hubs_normal,guint hubs_reg_vip,guint hubs_op,guint slots,guint speed_below_slot)
{
	if(userinfo != NULL && client_name != NULL && client_version != NULL)
	{
		userinfo->tag = g_strdup_printf((speed_below_slot > 0)?TAG_WITH_O:TAG_WITHOUT_O,client_name,client_version,mode,hubs_normal,hubs_reg_vip,hubs_op,slots,speed_below_slot);
	}
}

/* fonde i contenuti di <u2> con <u1>
 restituisce una nuova userinfo */
userinfo_t * userinfo_merge(userinfo_t * u1,userinfo_t * u2)
{
	userinfo_t * t = NULL;
	if(u1 != NULL && u2 != NULL)
	{
		t = g_malloc0(sizeof(userinfo_t));
		if(u2->nick != NULL) t->nick = g_strdup(u2->nick);
		else t->nick = g_strdup(u1->email);
		if(u2->email != NULL) t->email = g_strdup(u2->email);
		else t->email = g_strdup(u1->email);
		if(u2->client != NULL) t->client = g_strdup(u2->client);
		else t->client = g_strdup(u1->client);
		if(u2->tag != NULL) t->tag = g_strdup(u2->tag);
		else t->tag = g_strdup(u1->tag);
		if(u2->speed != NULL) t->speed = g_strdup(u2->speed);
		else t->speed = g_strdup(u1->speed);
		if(u2->desc != NULL) t->desc = g_strdup(u2->desc);
		else t->desc = g_strdup(u1->desc);
		if(u2->type != 0) t->type = u2->type;
		else t->type = u1->type;
		if(u2->is_shared == TRUE)
		{
			t->share_size = u2->share_size;
			t->is_shared = TRUE;
		}
		g_debug("(userinfo_merge): u1 is %s, u2 is %s",(u1->is_op==TRUE)?"operator":"normal",(u2->is_op==TRUE)?"operator":"normal");
		t->is_op = u2->is_op;
	}
	return t;
}

/* restituisce la modalita` di connessione del client (attivo/passivo) */
char userinfo_get_mode(userinfo_t * userinfo)
{
	if(userinfo != NULL)
	{
		if(userinfo->tag != NULL)
		{
			char * tmp = strstr(userinfo->tag,"M:A");
			if(tmp == NULL)
			{
				tmp = strstr(userinfo->tag,"M:P");
				if(tmp == NULL) return '\0';
			}
			return tmp[2];
		}
	}
	return '\0';
}

/* restituisce una copia delle userinfo <u> */
userinfo_t * userinfo_copy(userinfo_t * u)
{
	userinfo_t * c = NULL;
	if(u != NULL)
	{
		c = g_malloc0(sizeof(userinfo_t));
		memcpy(c,u,sizeof(userinfo_t));
		c->nick = g_strdup(u->nick);
		c->email = g_strdup(u->email);
		c->client = g_strdup(u->client);
		c->desc = g_strdup(u->desc);
		c->tag = g_strdup(u->tag);
		c->speed = g_strdup(u->speed);
	}
	return c;
}
